/// \file TiffFile.cpp
/// implements CImageFileDirectory class

/***************************************************************************
 *  Copyright (C) 2007 Pascal Corpet
 *  http://corpet.net/Pascal
 *
 *  This file is part of TiffRectoVerso.
 *
 *  TiffRectoVerso is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  TiffRectoVerso is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with TiffRectoVerso; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
***************************************************************************/

#include "ImageFileDirectory.h"

#include <crtdbg.h>

#define ASSERT(a) _ASSERT(a)
#if _DEBUG
	#define VERIFY(a) _ASSERT(a)
#else
	#define VERIFY(a) (a)
#endif

CImageFileDirectory::CImageFileDirectory(CTiffFile& i_tiff, CImageFileDirectory* i_previous)
{
	m_previous = i_previous;
	if (m_previous == NULL)
	{
		// the reference of the first IFD is at address 0x00000004 (it's in TIFF's header)
		m_reference = 0x00000004;
	}
	else
	{
		m_reference = i_previous->ReferenceNext(this);
	}

	// read address of directory
	i_tiff.SeekToAddress(m_reference);
	m_address = i_tiff.ReadAddress();

	if (m_address != 0)
	{
		// read size of directory

		// a directory contains the number of entries (on 2 bytes), then 12 bytes for each
		// entry and then the address of next IFD

		uint16 szNbEntries;
		i_tiff.SeekToAddress(m_address);
		szNbEntries = i_tiff.ReadUInt16();

		m_referenceNext = m_address + 2 + 12 * szNbEntries;
	}
	else
	{
		m_previous->m_next = NULL;
	}
}
bool CImageFileDirectory::End() const
{
	return m_address == 0;
}

uint32 CImageFileDirectory::ReferenceNext(CImageFileDirectory* i_next)
{
	ASSERT(!End());
	m_next = i_next;
	return m_referenceNext;
}

CImageFileDirectory* CImageFileDirectory::Previous()
{
	return m_previous;
}

bool CImageFileDirectory::InsertAfterMe(CTiffFile& i_tiff,CImageFileDirectory* i_nextOne)
{
	ASSERT(i_nextOne != NULL);
	ASSERT(m_next == NULL || m_next->m_previous == this);

	CImageFileDirectory* m_nextNext = m_next;

	// take out i_nextOne fromt the chain where it was

	if (i_nextOne->m_previous != NULL)
	{
		i_nextOne->m_previous->SetNext(i_tiff,i_nextOne->m_next);
	}

	if (i_nextOne->m_next != NULL)
	{
		i_nextOne->m_next->m_previous = i_nextOne->m_previous;
	}

	// insert i_nextOne in the chain 

	if (m_next != NULL)
	{
		m_next->m_previous = i_nextOne;
	}
	i_nextOne->SetNext(i_tiff,m_next);

	i_nextOne->m_previous = this;
	return SetNext(i_tiff,i_nextOne);
}

bool CImageFileDirectory::SetNext(CTiffFile& i_tiff, CImageFileDirectory* i_next)
{
	i_tiff.SeekToAddress(m_referenceNext);
	uint32 address = 0;
	if (i_next != NULL)
	{
		address = i_next->m_address;
	}
	m_next = i_next;
	return i_tiff.WriteAddress(address);
}
